package com.akamai.hdcore.samples.utility
{
	import com.akamai.hd.HDEvent;
	import com.akamai.hd.HDNetStream;
	
	import flash.display.StageDisplayState;
	import flash.net.NetConnection;
	import flash.net.NetStream;
	import flash.text.TextField;
	import flash.text.TextFieldAutoSize;
	
	import org.openvideoplayer.components.ui.shared.event.ControlEvent;
	import org.openvideoplayer.components.ui.shared.view.Label;
	import org.openvideoplayer.components.ui.shared.view.LabelButton;
	import org.openvideoplayer.utilities.TimeUtil;
	
	public class HDN1MediaPlayerViewPresenter extends BaseMediaPlayerViewPresenter
	{
		private var _showBufferMetrics:Boolean;
		
		public function HDN1MediaPlayerViewPresenter(view:MediaPlayerView, netStream:NetStream, netConnection:NetConnection,showBufferMetrics:Boolean = true)
		{
			_showBufferMetrics = showBufferMetrics;
			
			super(view, netStream, netConnection);
			setupNetStreamListeners();
		}
		
		override public function destroy():void
		{
			setupNetStreamListeners(false);
			super.destroy();
		}
		
		override protected function onSeekChange(event:ControlEvent):void
		{
			if (isSeeking && dualTimeCodeDisplay != null && !HDNetStream(netStream).isLiveStream)
			{
				var scrubTime:Number = scrubBar.getThumbPosition(HDNetStream(netStream).duration);
				dualTimeCodeDisplay.setTimeCodeText(TimeUtil.timeCode(scrubTime), HDNetStream(netStream).durationAsTimeCode);
			}
		}
		
		override protected function onSeekComplete(event:ControlEvent):void
		{
			if (!streamStarted)
			{
				scrubBar.setThumbPosition(0, HDNetStream(netStream).duration);
			}
			else
			{
				super.onSeekComplete(event);
				netStream.seek(scrubBar.getThumbPosition(HDNetStream(netStream).duration));
			}
		}
		
		override protected function setTimeDisplay():void
		{
			if (dualTimeCodeDisplay)
			{
				dualTimeCodeDisplay.setTimeCodeText(HDNetStream(netStream).timeAsTimeCode, HDNetStream(netStream).durationAsTimeCode);
			}
		}
		
		protected function setupNetStreamListeners(add:Boolean=true):void
		{
			if (add)
			{
				netStream.addEventListener(HDEvent.METADATA, onMetaData);
				netStream.addEventListener(HDEvent.PROGRESS, onProgress);
				netStream.addEventListener(HDEvent.IS_BUFFERING, onBuffer);				
				netStream.addEventListener(HDEvent.COMPLETE, onComplete);
			}
			else
			{
				netStream.removeEventListener(HDEvent.IS_BUFFERING, onBuffer);
				netStream.removeEventListener(HDEvent.METADATA, onMetaData);
				netStream.removeEventListener(HDEvent.PROGRESS, onProgress);
				netStream.removeEventListener(HDEvent.COMPLETE, onComplete);
			}
		}
		
		protected function onComplete(event:HDEvent):void
		{
			streamStarted = false;
			posterFrameButton.visible = true;
			playPauseButton.currentState = ControlEvent.PAUSE;
		}
		
		protected function onProgress(event:HDEvent):void
		{
			if(_showBufferMetrics)
			{
				updateBufferMetrics();
			}
			if (!isSeeking)
			{
				scrubBar.setThumbPosition(netStream.time, HDNetStream(netStream).duration);
				setTimeDisplay();
			}
		}
		
		private function onMetaData(event:HDEvent):void
		{
			if (event.data.width != MediaPlayerView(view).video.width || event.data.height != MediaPlayerView(view).video.height)
			{
				if (view.stage.displayState != StageDisplayState.FULL_SCREEN)
				{
					Utils.scaleVideo(MediaPlayerView(view), event.data.width, event.data.height);
				}
			}
		}
		
		private function onBuffer(event:HDEvent):void
		{
			event.data ? displaySpinner() : hideSpinner();
		}
		
		private function updateBufferMetrics():void
		{
			if (!view.captioningView.label.textfield.wordWrap)
			{
				view.captioningView.label.textfield.wordWrap = true;
				view.captioningView.label.textfield.autoSize = TextFieldAutoSize.LEFT;
			}
			
			view.captioningView.text = "trueBufferTime = "+Math.round(HDNetStream(netStream).trueBufferTime) +
			"  :  targetBufferTime = "+Math.round(netStream.bufferTime) +
			"  :  bufferLength = "+Math.round(netStream.bufferLength) +										
			"  :  downloadRatio = "+Math.round(HDNetStream(netStream).downloadRatio)+
			"  :  lastThrottleValue = "+Math.round(HDNetStream(netStream).lastThrottleValue) + 
			"  \n  droppedFrames = "+Math.round(HDNetStream(netStream).droppedFrames) +
			"  :  droppedFPS = "+Math.round(HDNetStream(netStream).droppedFPS);
			
			if (!view.captioningView.visible)
			{
				view.captioningView.visible = true;
			}
		}
	}
}