package com.akamai.player
{
	import com.akamai.hd.HDEvent;
	import com.akamai.hd.HDNetStream;
	
	import flash.display.*;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.geom.*;
	import flash.system.System;
	import flash.text.TextField;
	import flash.text.TextFormat;
	
	[ Event( name="fullScreen", type="flash.events.Event") ]
	[ Event( name="normal", type="flash.events.Event") ]
	[ Event( name="showstats", type="flash.events.Event") ]
	[ Event( name="waitForStream", type="flash.events.Event") ]
	[ Event( name="restart", type="flash.events.Event") ]
	
	public class Controller extends Sprite
	{
		private var ns:HDNetStream;
		private var isPlayingLive:Boolean = true;
		private var timeDisplay:TextField;
		private var dvrDisplay:TextField;
		private var backGround:Sprite;
		private var progress:Sprite;
		private var scrubBack:Sprite;
		private var volumeControl:VolumeControl;
		private var hdMeter:HDMeter;
		private var fullscreenButton:Button;
		private var statsButton:Button;
		private var pauseButton:Button;
		private var playButton:Button;
		private var volButton:Button;
		private var liveButton:Button;
		private var progressMaxWidth:Number;
		private var timeDisplayRightEdge:Number;
		private var errorMessage:String;
		private var closed:Boolean;
		
		[Embed(source='../../../assets/images/fullscreen.png')]
		private static var fullscreenPNG: Class;
		
		[Embed(source='../../../assets/images/stats.png')]
		private static var statsPNG: Class;
		
		[Embed(source='../../../assets/images/pause.png')]
		private static var pausePNG: Class;
		
		[Embed(source='../../../assets/images/play.png')]
		private static var playPNG: Class;
		
		[Embed(source='../../../assets/images/volume.png')]
		private static var volumePNG: Class;
		
		[Embed(source='../../../assets/images/golive.png')]
		private static var golivePNG: Class;
		
		
		public function Controller(width:Number,height:Number)
		{
			super();
			init(width,height);
		}
		public function setSize(width:Number,height:Number):void
		{
			createBackground(width,height);
			timeDisplayRightEdge = width-70;
			fullscreenButton.x = width-30;
			//statsButton.x = width-60;
			hdMeter.x = width-60;
			playButton.x = pauseButton.x = 0;
			volButton.x = volumeControl.x = 30;
			progress.x = scrubBack.x = 61;
			progress.y = scrubBack.y = 2;
			progressMaxWidth = width - 61 - 61;
			scrubBack.width = progressMaxWidth;
			volumeControl.y = -volumeControl.height+1;
		}
		public function setNetStream(netstream:HDNetStream):void
		{
			ns = netstream;
			ns.addEventListener(HDEvent.IS_PLAYING_LIVE,onIsPlayingLive);
			ns.addEventListener(HDEvent.PROGRESS,onProgress);
			ns.addEventListener(HDEvent.STREAM_NOT_FOUND,onStreamNotFound);
			ns.addEventListener(HDEvent.COMPLETE,onComplete);
			hdMeter.setNetStream(netstream);
		}
		
		public function displayError(msg:String):void
		{
			errorMessage = msg;
			timeDisplay.text  = errorMessage;
			timeDisplay.x = timeDisplayRightEdge - timeDisplay.textWidth;
			closed = true;
		}
		private function onComplete(event:HDEvent):void
		{
			ns.pause();
			closed = true;
			if (ns.isLiveStream)
			{
				// If the live stream is complete, we go into retry mode
				
				dispatchEvent(new Event("waitForStream"));
			}
			else
			{
				playButton.visible = true;
				pauseButton.visible = false;
			}
			
		}
		private function onProgress(event:HDEvent):void
		{
			
			if (errorMessage != "")
			{
				timeDisplay.text  = errorMessage;
				timeDisplay.x = timeDisplayRightEdge - timeDisplay.textWidth;
			}
			else
			{
				if (ns.time > 0)
				{
					closed = false;
					timeDisplay.text = isPlayingLive ? "LIVE":ns.secondsToTimeCode(ns.time)+" | "+ns.secondsToTimeCode(ns.duration);
				}
				else
				{
					timeDisplay.text = "Waiting";
				}
				timeDisplay.x = timeDisplayRightEdge - timeDisplay.textWidth;
				liveButton.x = timeDisplay.x  - 60;
				progress.width = (ns.time /ns.duration)*progressMaxWidth;
			}
		}
		private function onIsPlayingLive(event:HDEvent):void
		{
			isPlayingLive = event.data as Boolean;
			liveButton.visible = !isPlayingLive && ns.isLiveStream;
		}
		private function init(width:Number,height:Number):void
		{
			createChildren(width,height);
			setSize(width,height);
			errorMessage = "";
			closed = false;
		}
		private function createChildren(width:Number,height:Number):void
		{
			// Background
			backGround = new Sprite();
			addChild(backGround);
			//Fullscreen button
			fullscreenButton = new Button(30,30,new fullscreenPNG());
			fullscreenButton.addEventListener(MouseEvent.CLICK,onFullScreen);
			addChild(fullscreenButton);
			// HDmeter
			hdMeter = new HDMeter();
			hdMeter.addEventListener(MouseEvent.CLICK,onShowStats);
			addChild(hdMeter);
			//Pause button
			pauseButton = new Button(30,30,new pausePNG());
			pauseButton.addEventListener(MouseEvent.CLICK,onPause);
			addChild(pauseButton);
			//Play button
			playButton = new Button(30,30,new playPNG());
			playButton.addEventListener(MouseEvent.CLICK,onPlay);
			playButton.visible = false;
			addChild(playButton);
			//Volume button
			volButton = new Button(30,30,new volumePNG());
			volButton.addEventListener(MouseEvent.ROLL_OVER, onVolumeRollOver);
			volButton.addEventListener(MouseEvent.ROLL_OUT, onVolumeRollOut);
			addChild(volButton);
			// Scrub back
			scrubBack  = new Sprite();
			scrubBack.graphics.beginFill(0x000000,0);
			scrubBack.graphics.drawRect(0,0,1,height-4);
			scrubBack.addEventListener(MouseEvent.CLICK,onSeekClick);
			scrubBack.addEventListener(MouseEvent.ROLL_OVER,onRollOver);
			scrubBack.addEventListener(MouseEvent.ROLL_OUT,onRollOut);
			scrubBack.addEventListener(MouseEvent.MOUSE_MOVE,onMouseMove);
			scrubBack.useHandCursor = true;
			scrubBack.buttonMode = true;
			addChild(scrubBack);
			// Progress bar
			progress  = new Sprite();
			progress.graphics.beginFill(0x4a5787,1);
			progress.graphics.drawRect(0,0,1,height-4);
			progress.addEventListener(MouseEvent.CLICK,onSeekClick);
			progress.addEventListener(MouseEvent.ROLL_OVER,onRollOver);
			progress.addEventListener(MouseEvent.ROLL_OUT,onRollOut);
			progress.addEventListener(MouseEvent.MOUSE_MOVE,onMouseMove);
			progress.useHandCursor = true;
			progress.buttonMode = true;
			addChild(progress);
			// Dvr display
			dvrDisplay = new TextField();
			dvrDisplay.selectable = false;
			dvrDisplay.autoSize = "left";
			dvrDisplay.height=15;
			dvrDisplay.addEventListener(MouseEvent.CLICK,onSeekClick);
			dvrDisplay.addEventListener(MouseEvent.ROLL_OVER,onRollOver);
			dvrDisplay.addEventListener(MouseEvent.ROLL_OUT,onRollOut);
			dvrDisplay.addEventListener(MouseEvent.MOUSE_MOVE,onMouseMove);
			dvrDisplay.y = 6;
			dvrDisplay.x = 70;
			var tf2:TextFormat = new TextFormat("Arial",11,0xaaaaaa,true);
			dvrDisplay.defaultTextFormat = tf2;
			addChild(dvrDisplay);
			//Timedisplay
			timeDisplay = new TextField();
			timeDisplay.selectable = false;
			timeDisplay.autoSize = "right";
			timeDisplay.height=15;
			timeDisplay.addEventListener(MouseEvent.CLICK,onSeekClick);
			timeDisplay.addEventListener(MouseEvent.ROLL_OVER,onRollOver);
			timeDisplay.addEventListener(MouseEvent.ROLL_OUT,onRollOut);
			timeDisplay.addEventListener(MouseEvent.MOUSE_MOVE,onMouseMove);
			timeDisplay.y = 6;
			var tf:TextFormat = new TextFormat("Arial",11,0xFFFFFF,true);
			timeDisplay.defaultTextFormat = tf;
			addChild(timeDisplay);
			//live button
			liveButton = new Button(60,30,new golivePNG());
			liveButton.addEventListener(MouseEvent.CLICK,onGoLive);
			liveButton.visible = false;
			addChild(liveButton);
			// Volume control
			volumeControl = new VolumeControl();
			volumeControl.addEventListener("volumeChange",onVolumeChange);
			volumeControl.addEventListener(MouseEvent.ROLL_OVER, onVolumeRollOver);
			volumeControl.addEventListener(MouseEvent.ROLL_OUT, onVolumeRollOut);
			volumeControl.visible = false;
			addChild(volumeControl);
		}
		private function onVolumeChange(event:Event):void
		{
			ns.volume = volumeControl.volume;
		}
		private function onSeekClick(event:MouseEvent):void
		{
			if (!closed)
			{
				ns.seek((this.mouseX - 61)*ns.duration/progressMaxWidth);
			}
		}
		private function createBackground(width:Number,height:Number):void
		{
			if (backGround.numChildren >0)
			{
				backGround.removeChildAt(0);
			}
			var fType:String = GradientType.LINEAR;
			var colors:Array = [ 0x12297e, 0x1b2855];
			var alphas:Array = [ 1, 1 ];
			var ratios:Array = [ 0, 255 ];
			var matr:Matrix = new Matrix();
			matr.createGradientBox( width,height,Math.PI/2, 0, 0 );
			var sprMethod:String = SpreadMethod.PAD;
			var sprite:Sprite = new Sprite();
			var g:Graphics = sprite.graphics;
			g.beginGradientFill( fType, colors, alphas, ratios, matr, sprMethod );
			g.drawRect( 0, 0,width, height);
			g.endFill();
			g.lineStyle(1,0x000000);
			g.moveTo(0,0);
			g.lineTo(width-1,0);
			g.lineTo(width-1,height-1);
			g.lineTo(0,height-1);
			g.lineTo(0,0);
			g.moveTo(30,0);
			g.lineTo(30,30);
			g.moveTo(60,0);
			g.lineTo(60,30);
			g.moveTo(width-30,0);
			g.lineTo(width-30,30);
			g.moveTo(width-60,0);
			g.lineTo(width-60,30);
			backGround.addChild(sprite);
		}
		private function onFullScreen(event:MouseEvent):void
		{
			if (!closed)
			{
				dispatchEvent(new Event(this.stage.displayState));
			}
		}
		private function onShowStats(event:MouseEvent):void
		{
			if (!closed)
			{
				dispatchEvent(new Event("showstats"));
			}
		}
		private function onPause(event:MouseEvent):void
		{
			if (!closed)
			{
				playButton.visible = true;
				pauseButton.visible = false;
				ns.pause();
			}
		}
		private function onPlay(event:MouseEvent):void
		{
			playButton.visible = false;
			pauseButton.visible = true;
			dispatchEvent(new Event("resume"));
			ns.resume();
		}
		private function onVolumeRollOver(event:MouseEvent):void
		{
			if (!closed)
			{
				volumeControl.visible = true;
			}
			
		}
		private function onVolumeRollOut(event:MouseEvent):void
		{
			if (!closed)
			{
				volumeControl.visible = false;
			}
		}
		
		private function onGoLive(event:MouseEvent):void
		{
			ns.seek(-1);
		}

		private function onStreamNotFound(event:HDEvent):void
		{
			closed = true;
		}
		private function onRollOver(event:MouseEvent):void
		{
			dvrDisplay.visible = true;
		}
		private function onRollOut(event:MouseEvent):void
		{
			dvrDisplay.visible = false;
		}
		private function onMouseMove(event:MouseEvent):void
		{
			if (dvrDisplay.visible)
			{
				var time:Number = (this.mouseX - 61)*ns.duration/progressMaxWidth;
				dvrDisplay.text = (ns.isLiveStream ? "DVR to ":"SEEK to ") + ns.secondsToTimeCode(time);
			}
		}
	}
}